@@ -62,9 +62,6 @@ typedef struct {
     battery *batt;
     GdkPixbuf *plug;
     GdkPixbuf *flash;
-    GtkWidget *popup;               /* Popup message */
-    GtkWidget *alignment;           /* Alignment object in popup message */
-    GtkWidget *box;                 /* Vbox in popup message */
     guint timer;
     guint vtimer;
     gboolean pt_batt_avail;
@@ -85,14 +82,16 @@ typedef enum
 
 /* Prototypes */
 
-static void gtk_tooltip_window_style_set (PtBattPlugin *pt);
-static void draw_round_rect (cairo_t *cr, gdouble aspect, gdouble x, gdouble y, gdouble corner_radius, gdouble width, gdouble height);
-static void fill_background (GtkWidget *widget, cairo_t *cr, GdkColor *bg_color, GdkColor *border_color, guchar alpha);
-static void update_shape (PtBattPlugin *pt);
-static gboolean gtk_tooltip_paint_window (PtBattPlugin *pt);
-static gboolean ptbatt_window_click (GtkWidget * widget, GdkEventButton * event, PtBattPlugin *pt);
-static void show_message (PtBattPlugin *pt, char *str1, char *str2);
-static void hide_message (PtBattPlugin *pt);
+static gboolean is_pi (void);
+static void convert_alpha (guchar *dest_data, int dest_stride, guchar *src_data, int src_stride, int src_x, int src_y, int width, int height);
+GdkPixbuf *gdk_pixbuf_get_from_surface (cairo_surface_t *surface, gint src_x, gint src_y, gint width, gint height);
+static int init_measurement (PtBattPlugin *pt);
+static int charge_level (PtBattPlugin *pt, status_t *status, int *tim);
+static void draw_icon (PtBattPlugin *pt, int lev, float r, float g, float b, int powered);
+static void update_icon (PtBattPlugin *pt);
+static gboolean timer_event (PtBattPlugin *pt);
+static gboolean vtimer_event (PtBattPlugin *pt);
+
 
 static gboolean is_pi (void)
 {
@@ -102,163 +101,22 @@ static gboolean is_pi (void)
         return FALSE;
 }
 
-/* The functions below are a copy of those in GTK+2.0's gtktooltip.c, as for some reason, you cannot */
-/* manually cause a tooltip to appear with a simple function call. I have no idea why not... */
-
-static void on_screen_changed (GtkWidget *window)
+static int check_service (char *name)
 {
-	GdkScreen *screen;
-	GdkColormap *cmap;
+    int res;
+    char *buf;
 
-	screen = gtk_widget_get_screen (window);
-	cmap = NULL;
-	if (gdk_screen_is_composited (screen)) cmap = gdk_screen_get_rgba_colormap (screen);
-	if (cmap == NULL) cmap = gdk_screen_get_rgb_colormap (screen);
+    buf = g_strdup_printf ("systemctl status %s 2> /dev/null | grep -qw Active:", name);
+    res = system (buf);
+    g_free (buf);
 
-	gtk_widget_set_colormap (window, cmap);
-}
+    if (res) return 0;
 
-static void gtk_tooltip_window_style_set (PtBattPlugin *pt)
-{
-    gtk_alignment_set_padding (GTK_ALIGNMENT (pt->alignment), pt->popup->style->ythickness, pt->popup->style->ythickness,
-        pt->popup->style->xthickness, pt->popup->style->xthickness);
-    gtk_box_set_spacing (GTK_BOX (pt->box), pt->popup->style->xthickness);
-    gtk_widget_queue_draw (pt->popup);
-}
+    buf = g_strdup_printf ("systemctl status %s 2> /dev/null | grep -w Active: | grep -qw inactive", name);
+    res = system (buf);
+    g_free (buf);
 
-static void draw_round_rect (cairo_t *cr, gdouble aspect, gdouble x, gdouble y, gdouble corner_radius, gdouble width, gdouble height)
-{
-    gdouble radius = corner_radius / aspect;
-    cairo_move_to (cr, x + radius, y);
-    cairo_line_to (cr, x + width - radius, y);
-    cairo_arc (cr, x + width - radius, y + radius, radius, -90.0f * G_PI / 180.0f, 0.0f * G_PI / 180.0f);
-    cairo_line_to (cr, x + width, y + height - radius);
-    cairo_arc (cr, x + width - radius, y + height - radius, radius, 0.0f * G_PI / 180.0f, 90.0f * G_PI / 180.0f);
-    cairo_line_to (cr, x + radius, y + height);
-    cairo_arc (cr, x + radius, y + height - radius, radius,  90.0f * G_PI / 180.0f, 180.0f * G_PI / 180.0f);
-    cairo_line_to (cr, x, y + radius);
-    cairo_arc (cr, x + radius, y + radius, radius, 180.0f * G_PI / 180.0f, 270.0f * G_PI / 180.0f);
-    cairo_close_path (cr);
-}
-
-static void fill_background (GtkWidget *widget, cairo_t *cr, GdkColor *bg_color, GdkColor *border_color, guchar alpha)
-{
-    gint tooltip_radius;
-
-    if (!gtk_widget_is_composited (widget)) alpha = 255;
-
-    gtk_widget_style_get (widget, "tooltip-radius", &tooltip_radius,  NULL);
-
-    cairo_set_operator (cr, CAIRO_OPERATOR_CLEAR);
-    cairo_paint (cr);
-    cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
-
-    draw_round_rect (cr, 1.0, 0.5, 0.5, tooltip_radius, widget->allocation.width - 1,  widget->allocation.height - 1);
-
-    cairo_set_source_rgba (cr, (float) bg_color->red / 65535.0, (float) bg_color->green / 65535.0, (float) bg_color->blue / 65535.0, (float) alpha / 255.0);
-    cairo_fill_preserve (cr);
-
-    cairo_set_source_rgba (cr, (float) border_color->red / 65535.0, (float) border_color->green / 65535.0, (float) border_color->blue / 65535.0, (float) alpha / 255.0);
-    cairo_set_line_width (cr, 1.0);
-    cairo_stroke (cr);
-}
-
-static void update_shape (PtBattPlugin *pt)
-{
-    GdkBitmap *mask;
-    cairo_t *cr;
-    gint width, height, tooltip_radius;
-
-    gtk_widget_style_get (pt->popup, "tooltip-radius", &tooltip_radius, NULL);
-
-    if (tooltip_radius == 0 || gtk_widget_is_composited (pt->popup))
-    {
-        gtk_widget_shape_combine_mask (pt->popup, NULL, 0, 0);
-        return;
-    }
-
-    gtk_window_get_size (GTK_WINDOW (pt->popup), &width, &height);
-    mask = (GdkBitmap *) gdk_pixmap_new (NULL, width, height, 1);
-    cr = gdk_cairo_create (mask);
-
-    fill_background (pt->popup, cr, &pt->popup->style->black, &pt->popup->style->black, 255);
-    gtk_widget_shape_combine_mask (pt->popup, mask, 0, 0);
-
-    cairo_destroy (cr);
-    g_object_unref (mask);
-}
-
-static gboolean gtk_tooltip_paint_window (PtBattPlugin *pt)
-{
-    guchar tooltip_alpha;
-    gint tooltip_radius;
-
-    gtk_widget_style_get (pt->popup, "tooltip-alpha", &tooltip_alpha, "tooltip-radius", &tooltip_radius, NULL);
-
-    if (tooltip_alpha != 255 || tooltip_radius != 0)
-    {
-        cairo_t *cr = gdk_cairo_create (pt->popup->window);
-        fill_background (pt->popup, cr, &pt->popup->style->bg [GTK_STATE_NORMAL], &pt->popup->style->bg [GTK_STATE_SELECTED], tooltip_alpha);
-        cairo_destroy (cr);
-        update_shape (pt);
-    }
-    else gtk_paint_flat_box (pt->popup->style, pt->popup->window, GTK_STATE_NORMAL, GTK_SHADOW_OUT, NULL, pt->popup, "tooltip",
-        0, 0, pt->popup->allocation.width, pt->popup->allocation.height);
-
-    return FALSE;
-}
-
-static gboolean ptbatt_window_click (GtkWidget *widget, GdkEventButton *event, PtBattPlugin *pt)
-{
-    hide_message (pt);
-    return FALSE;
-}
-
-static void show_message (PtBattPlugin *pt, char *str1, char *str2)
-{
-    GtkWidget *item;
-    gint x, y;
-
-    if (pt->popup) return;
-
-    pt->popup = gtk_window_new (GTK_WINDOW_POPUP);
-    on_screen_changed (pt->popup);
-    gtk_window_set_type_hint (GTK_WINDOW (pt->popup), GDK_WINDOW_TYPE_HINT_TOOLTIP);
-    gtk_widget_set_app_paintable (pt->popup, TRUE);
-    gtk_window_set_resizable (GTK_WINDOW (pt->popup), FALSE);
-    gtk_widget_set_name (pt->popup, "gtk-tooltip");
-
-    pt->alignment = gtk_alignment_new (0.5, 0.5, 1.0, 1.0);
-    gtk_container_add (GTK_CONTAINER (pt->popup), pt->alignment);
-    gtk_widget_show (pt->alignment);
-
-    g_signal_connect_swapped (pt->popup, "style-set", G_CALLBACK (gtk_tooltip_window_style_set), pt);
-    g_signal_connect_swapped (pt->popup, "expose-event", G_CALLBACK (gtk_tooltip_paint_window), pt);
-
-    pt->box = gtk_vbox_new (FALSE, pt->popup->style->xthickness);
-    gtk_container_add (GTK_CONTAINER (pt->alignment), pt->box);
-
-    item = gtk_label_new (str1);
-    gtk_box_pack_start (GTK_BOX (pt->box), item, FALSE, FALSE, 0);
-    item = gtk_label_new (str2);
-    gtk_box_pack_start (GTK_BOX (pt->box), item, FALSE, FALSE, 0);
-
-    gtk_widget_show_all (pt->popup);
-    gtk_widget_hide (pt->popup);
-    lxpanel_plugin_popup_set_position_helper (pt->panel, pt->plugin, pt->popup, &x, &y);
-    gdk_window_move (gtk_widget_get_window (pt->popup), x, y);
-    gdk_window_set_events (gtk_widget_get_window (pt->popup), gdk_window_get_events (gtk_widget_get_window (pt->popup)) | GDK_BUTTON_PRESS_MASK);
-    g_signal_connect (G_OBJECT(pt->popup), "button-press-event", G_CALLBACK (ptbatt_window_click), pt);
-    gtk_window_present (GTK_WINDOW (pt->popup));
-}
-
-static void hide_message (PtBattPlugin *pt)
-{
-    if (pt->popup)
-    {
-		gtk_widget_destroy (pt->popup);
-		pt->popup = NULL;
-	}
+    return res;
 }
 
 /* gdk_pixbuf_get_from_surface function from GDK+3 */
@@ -337,9 +195,9 @@ static int init_measurement (PtBattPlugin *pt)
         pt->requester = NULL;
         pt->context = NULL;
         pt->pt_batt_avail = FALSE;
-        if (system ("systemctl status pt-device-manager | grep -wq active") == 0)
+        if (check_service ("pt-device-manager") || check_service ("pi-topd"))
         {
-            g_message ("pi-top device manager found");
+            g_message ("ptbatt: pi-top device manager found");
             pt->context = zmq_ctx_new ();
             if (pt->context)
             {
@@ -350,7 +208,7 @@ static int init_measurement (PtBattPlugin *pt)
                     {
                         if (zmq_send (pt->requester, "118", 3, ZMQ_NOBLOCK) == 3)
                         {
-                            g_message ("connected to pi-top device manager");
+                            g_message ("ptbatt: connected to pi-top device manager");
                             pt->pt_batt_avail = TRUE;
                             return 1;
                         }
@@ -585,7 +443,7 @@ static gboolean vtimer_event (PtBattPlugin *pt)
         int val = fgetc (fp);
         fclose (fp);
         if (val == '1')
-            show_message (pt, _("Low voltage warning"), _("Please check your power supply"));
+            lxpanel_notify (pt->panel, _("Low voltage warning\nPlease check your power supply"));
     }
     return TRUE;
 }
@@ -596,7 +454,8 @@ static gboolean vtimer_event (PtBattPlugin *pt)
 static void ptbatt_configuration_changed (LXPanel *panel, GtkWidget *p)
 {
     PtBattPlugin *pt = lxpanel_plugin_get_data (p);
-    update_icon (pt);
+    if (pt->timer) update_icon (pt);
+    else gtk_widget_hide (pt->plugin);
 }
 
 /* Plugin destructor. */
@@ -622,28 +481,27 @@ static GtkWidget *ptbatt_constructor (LXPanel *panel, config_setting_t *settings
 {
     /* Allocate and initialize plugin context */
     PtBattPlugin *pt = g_new0 (PtBattPlugin, 1);
-    pt->panel = panel;
-    pt->settings = settings;
 
 #ifdef ENABLE_NLS
     setlocale (LC_ALL, "");
     bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
     bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
-    textdomain (GETTEXT_PACKAGE);
 #endif
 
+    /* Allocate top level widget and set into plugin widget pointer. */
+    pt->panel = panel;
+    pt->settings = settings;
+    pt->plugin = gtk_event_box_new ();
+    lxpanel_plugin_set_data (pt->plugin, pt, ptbatt_destructor);
+
+    /* Allocate icon as a child of top level */
+    pt->tray_icon = gtk_image_new ();
+    gtk_container_add (GTK_CONTAINER (pt->plugin), pt->tray_icon);
+
     pt->ispi = is_pi ();
 
     if (init_measurement (pt))
     {
-        /* Allocate top level widget and set into Plugin widget pointer. */
-        pt->plugin = gtk_event_box_new ();
-
-        /* Allocate icon as a child of top level */
-        pt->tray_icon = gtk_image_new ();
-        gtk_widget_set_visible (pt->tray_icon, TRUE);
-        gtk_container_add (GTK_CONTAINER (pt->plugin), pt->tray_icon);
-
         /* Load the symbols */
         pt->plug = gdk_pixbuf_new_from_file ("/usr/share/lxpanel/images/plug.png", NULL);
         pt->flash = gdk_pixbuf_new_from_file ("/usr/share/lxpanel/images/flash.png", NULL);
@@ -651,17 +509,13 @@ static GtkWidget *ptbatt_constructor (LXPanel *panel, config_setting_t *settings
         /* Start timed events to monitor status */
         pt->timer = g_timeout_add (INTERVAL, (GSourceFunc) timer_event, (gpointer) pt);
     }
-    else
-    {
-        pt->timer = 0;
-        /* a NULL label has a width of zero; unlike an empty button... */
-        pt->plugin = gtk_label_new (NULL);
-    }
+    else pt->timer = 0;
 
     /* Start timed events to monitor low voltage warnings */
     if (pt->ispi) pt->vtimer = g_timeout_add (VMON_INTERVAL, (GSourceFunc) vtimer_event, (gpointer) pt);
 
-    lxpanel_plugin_set_data (pt->plugin, pt, ptbatt_destructor);
+    /* Show the widget and return */
+    gtk_widget_show_all (pt->plugin);
     return pt->plugin;
 }
 
